@@ -690,7 +690,7 @@ def load(tag, creds=None, session_creds=True) -> Dataset:
     fs, path = _load_fs_and_path(tag, creds, session_creds=session_creds)
     fs: fsspec.AbstractFileSystem = fs
     path_2 = f"{path}/meta.json"
-    if not fs.exists(path):
+    if not fs.exists(path_2):
         raise HubDatasetNotFoundException(tag)
 
     with fs.open(path_2, "r") as f:
