@@ -4,10 +4,10 @@ from hub.schema.features import Tensor
 
 
 class BBox(Tensor):
-    """`HubSchema` for a normalized bounding box.
-    Output:
-    bbox: Tensor of type `float32` and shape `[4,]` which contains the
-          normalized coordinates of the bounding box `[ymin, xmin, ymax, xmax]`
+    """| HubSchema` for a normalized bounding box.
+
+    Output: Tensor of type `float32` and shape `[4,]` which contains the
+    normalized coordinates of the bounding box `[ymin, xmin, ymax, xmax]`
     """
 
     def __init__(self, dtype="float64", chunks=None, compressor="lz4"):
